<?php

namespace Drupal\Tests\csv_field\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the csv field widget.
 *
 * @group file
 */
class CsvFieldTest extends WebDriverTestBase {

  /**
   * An user with administration permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['node', 'file', 'csv_field_test'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->adminUser = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'administer nodes',
      'bypass node access',
    ]);
    $this->drupalLogin($this->adminUser);

    $this->fileSystem = $this->container->get('file_system');
  }

  /**
   * Tests upload and display of CSV field.
   */
  public function testSingleValuedDatatable() {
    $page = $this->getSession()->getPage();
    $assert_session = $this->assertSession();

    $this->drupalGet("node/add/blog_post");

    $title = 'Testing Datatables';

    $page->findField('title[0][value]')->setValue($title);

    $test_file_path = $this->root . '/' . drupal_get_path('module', 'csv_field_test') . '/assets/addresses.csv';
    $page->attachFileToField('files[field_csv_file_0]', $test_file_path);
    $expected_element = ['id_or_name', 'field_csv_file_0_remove_button'];
    $this->assertSession()->waitForElement('named', $expected_element);


    $this->openConfigurationDetails();

    $this->assertTrue($this->assertSession()->optionExists('field_csv_file[0][settings][pageLength]', '10')->isSelected());
    $assert_session->checkboxChecked('field_csv_file[0][settings][lengthChange]');
    $assert_session->checkboxChecked('field_csv_file[0][settings][searching]');
    $assert_session->checkboxChecked('field_csv_file[0][settings][download]');
    $assert_session->fieldValueEquals('field_csv_file[0][settings][responsive]', 'childRow');
    $page->fillField('field_csv_file[0][settings][downloadText]', 'Addresses Description');

    $page->pressButton('Save');

    // Assert the wrapper div has the expected settings.
    $div_with_settings = $assert_session->elementExists('css', 'div.dataTable');
    $settings = $div_with_settings->getAttribute('data-settings');
    $this->assertStringContainsString('"paging":1', $settings);
    $this->assertStringContainsString('"pageLength":10', $settings);
    $this->assertStringContainsString('"lengthChange":1', $settings);
    $this->assertStringContainsString('"searching":1', $settings);
    $this->assertStringContainsString('"responsive":"childRow"', $settings);
    $this->assertStringContainsString('"download":1', $settings);

    $this->assertSession()->waitForElement('css', '.dataTables_wrapper');

    // This element is added by the datatables JavaScript plugin, so if it's
    // present that means the table has been rendered by the plugin.
    $this->assertSession()->elementExists('css', '.dataTables_wrapper');

    // Assert the Search filter field is present.
    $assert_session->elementExists('css', '.dataTables_filter input[type=search]');
    // Assert the Row length change select is present.
    $assert_session->elementExists('css', '.dataTables_length select');
    // Assert the pagination div is present.
    $assert_session->elementExists('css', '.dataTables_paginate');
    // Assert download link is present.
    $this->assertSession()->linkExists('Addresses Description');

    $node = $this->drupalGetNodeByTitle($title);
    $edit_path = sprintf('node/%d/edit', $node->id());

    $this->drupalGet($edit_path);

    $this->openConfigurationDetails();

    $page->selectFieldOption('field_csv_file[0][settings][pageLength]', '25');
    $page->selectFieldOption('field_csv_file[0][settings][responsive]', 'childRowImmediate');
    $page->uncheckField('field_csv_file[0][settings][searching]');
    $page->uncheckField('field_csv_file[0][settings][download]');
    $page->uncheckField('field_csv_file[0][settings][paging]');

    $page->pressButton('Save');

    // Assert the wrapper div has the expected settings.
    // Assert the wrapper div has the expected settings.
    $div_with_settings = $assert_session->elementExists('css', 'div.dataTable');
    $settings = $div_with_settings->getAttribute('data-settings');
    $this->assertStringContainsString('"paging":0', $settings);
    $this->assertStringNotContainsString('pageLength', $settings);
    $this->assertStringNotContainsString('lengthChange', $settings);
    $this->assertStringContainsString('"searching":0', $settings);
    $this->assertStringContainsString('"responsive":"childRowImmediate"', $settings);
    $this->assertStringContainsString('"download":0', $settings);
    $this->assertStringNotContainsString('downloadText', $settings);

    $this->assertSession()->waitForElement('css', '.dataTables_wrapper');

    // Assert the Search filter field is not present.
    $assert_session->elementNotExists('css', '.dataTables_filter input[type=search]');
    // Assert the Row length change select is not present.
    $assert_session->elementNotExists('css', '.dataTables_length select');
    // Assert the pagination div is not present.
    $assert_session->elementNotExists('css', '.dataTables_paginate');
    // Assert download link is not present.
    $this->assertSession()->linkNotExists('Addresses Description');
  }

  /**
   * Helper function to open the configuration details.
   */
  protected function openConfigurationDetails() {
    $page = $this->getSession()->getPage();
    $assert_session = $this->assertSession();

    // Assert Configuration details is closed.
    $paging_checkbox = $assert_session->fieldExists('field_csv_file[0][settings][paging]');
    $this->assertFalse($paging_checkbox->isVisible());

    // Open Configuration details.
    $assert_session->elementExists('xpath', '//details/summary[text()="Display Configuration"]')->click();
    // Assert Configuration details is open.
    $page->waitFor(10, function () use ($paging_checkbox) {
      return $paging_checkbox->isVisible();
    });
    $this->assertTrue($paging_checkbox->isVisible());
  }

}
